/***************************************************************************
 *   Copyright (C) 2005 by Johnathan Burchill   *
 *   jkerrb@shaw.ca   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "kdarcontroller.h"

#include "../kdar_part.h"

#include "archivehandle.h"
#include "kdardeclarations.h"
#include "kdarenums.h"
#include "kdarinteraction.h"

#include "threads/openarchivethread.h"
#include "threads/testarchivethread.h"

#include "dialogs/kdaropenarchivedialog.h"

#include "events/kdardeletethreadevent.h"
#include "events/kdararchivebrowserupdatesevent.h"
#include "events/kdarcheckfilesizeevent.h"
#include "events/kdarcancelbuttonevent.h"
#include "events/kdarlogfilecommandevent.h"
#include "events/kdarpostlistviewitemevent.h"
#include "events/kdarprogressevent.h"
#include "events/kdarprogressdialogevent.h"
#include "events/kdarsetcursorevent.h"
#include "events/kdarstatusbarevent.h"
#include "events/kdartarlistingevent.h"
#include "events/kdaruserinteractionevent.h"
#include "events/kdaruserinteractionstringevent.h"
#include "events/kdarwarningevent.h"

#include "../kdardockwidgets/kdarbrowser.h"
#include "../kdardockwidgets/kdarlistviewitem.h"

#include <kdebug.h>
//#include <kfiledialog.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kpassdlg.h>

#include <qapplication.h>
#include <qevent.h>
#include <qpushbutton.h>
#include <qsemaphore.h>

#include <dar/catalogue.hpp>

#include <iostream>

KDarController::KDarController( QObject* parent, const char* name )
    : m_parent( parent )
{
    setName( name );
    //Initialize pointers to zero:
    KDarPart* part = static_cast<KDarPart*> ( m_parent );
    if ( part )
    {
        m_browser = (KDarBrowser*) part->browserDockWidget();
    }
    else
    {
        m_browser = 0;
    }

    //The dictionary of open archives.
    m_archives = new QPtrDict<QString>( 17 );
    m_archives->setAutoDelete( true );
    //The dictionary of running threads. Size is 17. Should be large enough, right?
    //The size needs to be a prime number to keep this efficient.
    m_threads = new QPtrDict<QString>( 17 );
    m_threads->setAutoDelete( true );

    m_createThread = 0;
    m_diffThread = 0;
    m_isolateThread = 0;
    m_listThread = 0;
    m_openThread = 0;
    m_restoreThread = 0;
    m_testThread = 0;
    m_localWallet = 0;
    m_password = "";
    m_userResponse = false;
    //The semaphore lets us pause and wake threads during user interaction...
    m_semaphore = new QSemaphore( 1 );
    ( *m_semaphore )++;
    //Set up the libdar capabilities info
    libdar::get_compile_time_features( m_handlesEA, m_handlesLargefiles, m_handlesNoDump, m_usesSpecialAlloc, m_integerSizeBits, m_isThreadSafe, m_handlesZCompression, m_handlesBZ2Compression, m_handlesStrongCrypto );
    //libdar version
    libdar::U_I maj, med, min;
    libdar::get_version(maj, med, min);
    m_libdarVersion = i18n( "%1.%2.%3" ).arg( libdar::deci( maj ).human().c_str() ).arg( libdar::deci( med ).human().c_str() ).arg( libdar::deci( min ).human().c_str() );
    m_browserArchive = 0;

    QWidget* widget = ((KDarPart*) m_parent)->widget();
    m_openDialog = new KDarOpenArchiveDialog( KDARDIRKEY, "*.1.dar", widget, i18n( "Select archive to open" ), true );

}

KDarController::~KDarController()
{
    if ( m_openDialog )
        delete m_openDialog;
    //TODO Stop each thread cleanly!
    if ( m_threads )
        delete m_threads;
    //TODO Close each archive cleanly!
    if ( m_archives )
        delete m_archives;
}

void KDarController::setArchive( libdar::archive* archive, const QString & path )
{
    m_browserArchive = archive;
    //We can have several archives open at once.
    if ( !m_archives->find( archive ) && !path.isEmpty() )
    {
        m_archives->insert( archive, new QString( path ) );
        archiveHandle handle( path );
        m_browser->addArchive( archive, path, handle.basename() );
    }
}

libdar::archive* KDarController::archive() const
{
    return m_browserArchive;
}

void KDarController::closeArchive()
{
}

void KDarController::createArchive( bool updateListView )
{
    std::cout << "KDarController::createArchive(): need to implement" << std::endl;
    std::cout << "KDarController::createArchive(): updateListView is " << updateListView << std::endl;
}

void KDarController::diffArchive()
{
    std::cout << "KDarController::diffArchive(): need to implement" << std::endl;
}

void KDarController::isolateArchive()
{
    std::cout << "KDarController::isolateArchive(): need to implement" << std::endl;
}

void KDarController::listArchive( bool )
{
    std::cout << "KDarController::listArchive(): need to implement" << std::endl;
}

void KDarController::openArchive( const QString & name, bool updateListView )
{
    //Get the filename to open
    QString filename( name );
    QString inputPipe( "" );
    QString outputPipe( "" );
    QString executeCommand( "" );
    bool verbose( false );
    kdDebug() << "KDarController::openArchive( " << filename << " )" << endl;
    if ( filename.isEmpty() && m_openDialog )
    {
        int result = m_openDialog->exec();
        if ( result == QDialog::Accepted )
        {
            filename = m_openDialog->selectedFile();
            inputPipe = m_openDialog->inputPipe();
            outputPipe = m_openDialog->outputPipe();
            executeCommand = m_openDialog->executeCommand();
            verbose = m_openDialog->verbose();
        }
    }
    if ( !filename.isEmpty() )
    {
        archiveHandle handle( filename );
        emit openedArchive( handle.archiveName() );

        if ( updateListView )
        {
            m_browser->clearBrowserView();
        }
        KDarThread* openThread = new OpenArchiveThread( this, filename, inputPipe, outputPipe, executeCommand, verbose );
        m_threads->insert( openThread, new QString( "Opening " + filename ) );
        openThread->start();
    }
}

void KDarController::restoreArchive()
{
    std::cout << "KDarController::restoreArchive(): need to implement" << std::endl;
}

void KDarController::testArchive()
{
    libdar::archive* theArchive = m_browserArchive;
    if ( theArchive )
    {
        bool verbose( true );
        kdDebug() << "KDarController::testArchive()" << endl;
        bool showSkipped = true;
        KDarThread* testThread = new TestArchiveThread( this, theArchive, verbose, showSkipped );
        m_threads->insert( testThread, new QString( "Testing " + *( m_archives->find( theArchive ) ) ) );
        testThread->start();
    }
}

void KDarController::openLogFile( const QString & )
{
}

void KDarController::closeLogFile()
{
}

void KDarController::waitForThread( threadType type )
{
    kdDebug() << "KDarController::waitForThread(): thread type is " << type << endl;
    if ( type == OPEN_THREAD )
    {
        kdDebug() << "KDarController::waitForThread(): open thread." << endl;
    }
}

void KDarController::customEvent( QCustomEvent * event )
{
    KDarPart* part = static_cast<KDarPart*> ( m_parent );
    if ( event->type() == (QEvent::Type) KDar::SETCURSOR )
    {
        if ( m_parent != NULL && m_parent != 0 )
        {
            kdarSetCursorEvent* sce = (kdarSetCursorEvent*)event;
            kdarSetCursorEvent* newsce = new kdarSetCursorEvent( sce->cursor() );
            QApplication::postEvent( m_parent, newsce );
        }
    }
    if ( event->type() == ( QEvent::Type ) KDar::CHECKFILESIZE )
    {
        if ( m_parent != NULL && m_parent != 0 )
        {
            kdarCheckFileSizeEvent *cfse = (kdarCheckFileSizeEvent*) event;
            kdarCheckFileSizeEvent *newcfse = new kdarCheckFileSizeEvent( cfse->checkFileSizeAction() );
            QApplication::postEvent( m_parent, newcfse );
        }
        else
        {
            kdDebug() << "KDarController::customEvent(): m_parent isn't right!" << endl;
        }
    }
    if ( event->type() == ( QEvent::Type ) KDar::PROGRESS )
    {
        if ( m_parent != NULL && m_parent != 0 )
        {
            kdarProgressEvent *pe = (kdarProgressEvent*) event;
            kdarProgressEvent *newpe = new kdarProgressEvent( pe->command() );
            QApplication::postEvent( m_parent, newpe );
        }
        else
        {
            kdDebug() << "KDarController::customEvent(): m_parent isn't right!" << endl;
        }
    }
    if ( event->type() == ( QEvent::Type ) KDar::STATUSBAR )
    {
        if ( m_parent != NULL && m_parent != 0 )
        {
            kdarStatusBarEvent *se = (kdarStatusBarEvent*) event;
            //Fix this output: There are spaces between lines.
//            QString message = se->message();
//            writeLogMessage( message );
            KDarPart* part = static_cast<KDarPart*>(m_parent);
            if ( part )
            {
                part->setStatusBarMessage( se->message(), se->duration() );
            }
//             kdarStatusBarEvent *newse = new kdarStatusBarEvent( se->message(), se->duration() );
//             QApplication::postEvent( m_parent, newse );
        }
        else
        {
            kdDebug() << "KDarController::customEvent(): m_parent isn't right!" << endl;
        }
    }
    if ( event->type() == ( QEvent::Type ) KDar::PROGRESSDIALOG )
    {
        if ( m_parent != NULL && m_parent != 0 )
        {
            kdarProgressDialogEvent *pde = (kdarProgressDialogEvent*) event;
            kdarProgressDialogEvent *newpde = new kdarProgressDialogEvent( pde->message(), pde->field() );
            QApplication::postEvent( m_parent, newpde );
        }
        else
        {
            kdDebug() << "KDarController::customEvent(): m_parent isn't right!" << endl;
        }
    }
    if ( event->type() == ( QEvent::Type ) KDar::WARNING )
    {
        if ( m_parent != NULL && m_parent != 0 )
        {
            kdarWarningEvent *we = (kdarWarningEvent*) event;
            QString message = we->message();
            kdDebug() << "WARNING: " << message << endl;
            writeLogMessage( message );
        }
        else
        {
            kdDebug() << "KDarController::customEvent(): m_parent isn't right!" << endl;
        }
    }
    if ( event->type() == ( QEvent::Type ) KDar::TARLISTING )
    {
        kdarTarListingEvent *tle = (kdarTarListingEvent*) event;
//        kdDebug() << "TARLISTING: " << tle->flag() << tle->filename() << endl;
        if ( m_browser )
        {
            m_browser->addBrowserEntry( tle->filename(), tle->perm(), tle->uid(), tle->gid(), tle->size(), tle->date(), tle->flag(), tle->isDir(), tle->hasChildren() );
        }
    }
    if ( event->type() == ( QEvent::Type ) KDar::USERINTERACTION )
    {
        kdDebug() << "KDarController::customEvent(): USERINTERACTION" << endl;
        kdarUserInteractionEvent *uie = (kdarUserInteractionEvent*) event;
        QString question = uie->question();
        int ret = KMessageBox::warningYesNo(0, question );
        bool response = (ret == KMessageBox::Yes);
        setUserResponse( response );
        if ( m_semaphore )
        {
            (*m_semaphore)--;
        }
    }
    if ( event->type() == ( QEvent::Type ) KDar::USERINTERACTIONSTRING )
    {
        kdarUserInteractionStringEvent *uise = (kdarUserInteractionStringEvent*) event;
        QString question = uise->question();
        bool echo = uise->echo();
        //TODO: We should not assume that libdar wants a password...
        if ( !echo )
        {
            bool gotPassword = obtainPassword();
//            QString answer = i18n( "Did libdar get a password from the user?", "did not get password.\n" );
//            if ( gotPassword ) QString answer = i18n( "Did libdar get a password from the user?", "got password.\n" );
//            logMessage = i18n( "(Libdar request for text)...(user\'s reply)","%1...%2\n" ).arg( question ).arg( answer );
        }
        if ( m_semaphore )
        {
            (*m_semaphore)--;
        }
    }
    if ( event->type() == ( QEvent::Type ) KDar::POSTLISTVIEWITEM )
    {
        if ( m_parent != NULL && m_parent != 0 )
        {
            kdarPostListViewItemEvent *pe = (kdarPostListViewItemEvent*) event;
            kdarPostListViewItemEvent *newpe = new kdarPostListViewItemEvent( pe->text(), pe->clearView() );
            QApplication::postEvent( m_parent, newpe );
        }
        else
        {
            kdDebug() << "KDarController::customEvent(): m_parent isn't right!" << endl;
        }
    }
    if ( event->type() == ( QEvent::Type ) KDar::ARCHIVEBROWSERUPDATES )
    {
        if ( m_parent != NULL && m_parent != 0 )
        {
            kdarArchiveBrowserUpdatesEvent *ae = (kdarArchiveBrowserUpdatesEvent*) event;
            kdarArchiveBrowserUpdatesEvent *newae = new kdarArchiveBrowserUpdatesEvent( ae->command() );
            QApplication::postEvent( m_parent, newae );
        }
        else
        {
            kdDebug() << "KDarController::customEvent(): m_parent isn't right!" << endl;
        }
    }
    if ( event->type() == ( QEvent::Type ) KDar::DELETETHREAD )
    {
        kdarDeleteThreadEvent* dte = (kdarDeleteThreadEvent*)event;
        KDarThread* thread = (KDarThread*) dte->thread();
        if ( !thread ) return;
        if ( thread->running() )
        {
            thread->wait();
        }
        kdDebug() << "KDarController::customEvent( DELETETHREAD ): deleting " << *( m_threads->find( thread ) )<< endl;
        //This automatically deletes the item
        m_threads->remove( thread );

/*        //set the corresponding thread pointer to zero
        if ( (createArchiveThread*) thread == m_createThread )
        {
            m_createThread = 0;
            kdDebug() << "KDarController::customEvent(): deleted m_createThread" << endl;
        }
        if ( (diffArchiveThread*) thread == m_diffThread )
        {
            m_diffThread = 0;
            kdDebug() << "KDarController::customEvent(): deleted m_diffThread" << endl;
        }
        if ( (isolateArchiveThread*) thread == m_isolateThread )
        {
            m_isolateThread = 0;
            kdDebug() << "KDarController::customEvent(): deleted m_isolateThread" << endl;
        }
        if ( (listArchiveThread*) thread == m_listThread )
        {
            m_listThread = 0;
            kdDebug() << "KDarController::customEvent(): deleted m_listThread" << endl;
        }
        if ( (OpenArchiveThread*) thread == m_openThread )
        {
            m_openThread = 0;
            //kdDebug() << "KDarController::customEvent(): deleted m_openThread" << endl;
        }
        if ( (restoreArchiveThread*) thread == m_restoreThread )
        {
            m_restoreThread = 0;
            kdDebug() << "KDarController::customEvent(): deleted m_restoreThread" << endl;
        }
        if ( (TestArchiveThread*) thread == m_testThread )
        {
            m_testThread = 0;
            kdDebug() << "KDarController::customEvent(): deleted m_testThread" << endl;
        }*/
    }
    if ( event->type() == (QEvent::Type) KDar::CANCELBUTTON )
    {
        if ( part )
        {
            kdarCancelButtonEvent* cbe = (kdarCancelButtonEvent*)event;
            part->setCancelState( cbe->state() );
        }
    }
    if ( event->type() == (QEvent::Type) KDar::LOGFILECOMMAND )
    {
        kdarLogFileCommandEvent* lce = (kdarLogFileCommandEvent*)event;
        QString logText = lce->text();
        switch ( (int) lce->command() )
        {
            case kdarLogFileCommandEvent::OPEN:
                openLogFile( logText );
                break;
            case kdarLogFileCommandEvent::CLOSE:
                closeLogFile();
                break;
            case kdarLogFileCommandEvent::WRITEMESSAGE:
                writeLogMessage( logText );
                break;
            case kdarLogFileCommandEvent::FLUSH:
                m_logFile.flush();
            default:
                ;
        }
    }
}

void KDarController::writeLogMessage( QString & )
{
}

bool KDarController::cancelOperation()
{
    return false;
}

bool KDarController::openKDEWallet()
{
    return false;
}

bool KDarController::obtainPassword()
{
    kdDebug() << "KDarController::obtainPassword(): called." << endl;
    bool success = true;
    bool askUserForPassword = false;
    bool usingWallet = false;

/*    bool usingWallet = true;
    if ( m_parent != NULL && m_parent !=0 )
    {
        usingWallet = m_parent->useWallet();
    }
*/
    if ( ! m_localWallet || ! usingWallet )
    {
        //No wallet
        askUserForPassword = true;
        kdDebug() << "KDarController::obtainPassword(): not using wallet." << endl;
    }
    //Ask user for password?
    if ( askUserForPassword )
    {
        kdDebug() << "KDarController::obtainPassword(): asking user for password." << endl;
        int result = KPasswordDialog::Rejected;
        QCString userPassword;
        result = KPasswordDialog::getPassword( userPassword, i18n( "Please enter the password:" ) );
        if ( result == KPasswordDialog::Accepted )
        {
            m_password = ( QString ) userPassword;
        }
        else
        {
            m_password = QString( "" );
            success = false;
        }
    }
    return success;
}

QString KDarController::password()
{
    return m_password;
}

void KDarController::clearPassword()
{
    //Clear the password from memory...Does this actually work?
    m_password = "";
}

bool KDarController::changePassword()
{
    return false;
}

void KDarController::setUserResponse( const bool )
{
}

bool KDarController::userResponse()
{
    return false;
}

QSemaphore* KDarController::semaphore()
{
    return m_semaphore;
}

bool KDarController::handlesEA() const
{
    return m_handlesEA;
}

bool KDarController::handlesLargefiles() const
{
    return m_handlesLargefiles;
}

bool KDarController::handlesNoDump() const
{
    return m_handlesNoDump;
}

bool KDarController::usesSpecialAlloc() const
{
    return m_usesSpecialAlloc;
}

libdar::U_I KDarController::integerSizeBits() const
{
    return m_integerSizeBits;
}

bool KDarController::isThreadSafe() const
{
    return m_isThreadSafe;
}

bool KDarController::handlesZCompression() const
{
    return m_handlesZCompression;
}

bool KDarController::handlesBZ2Compression() const
{
    return m_handlesBZ2Compression;
}

bool KDarController::handlesStrongCrypto() const
{
    return m_handlesStrongCrypto;
}

const QString KDarController::libdarVersion() const
{
    return m_libdarVersion;
}

KURL KDarController::extractItem( const QString & path )
{
    KURL url( "" );
    if ( !path.isEmpty() )
    {
//         kdDebug() << "KDarController::extractItem(): path = " << path << endl;
        KDarInteraction ki( this );
        if ( m_browserArchive )
        {
            try
            {
                QString root="/tmp";
//                 kdDebug() << "KDarController::extractItem(): op_extract()" << endl;
                m_browserArchive->op_extract( ki, root.latin1(), libdar::bool_mask( true ), libdar::simple_path_mask( root + "/" + path, true ), true, false, true, false, false, libdar::bool_mask( true ), false, libdar::inode::cf_all, false, 0, false, false, false, 0 );
                url = root + "/" + path;
//                 kdDebug() << "KDarController::extractItem(): extracted(). url is " << url << endl;
            }
            catch( libdar::Egeneric & e )
            {
//                 kdDebug() << "KDarController::extractItem(): " << e.get_message() << endl;

            }
        }
    }
    return url;
}

void KDarController::slotGetChildren( QString & path )
{
//     kdDebug() << "KDarController::getChildren()" << endl;
    if ( ! path.isEmpty() )
    {
//         kdDebug() << "KDarController::getChildren(): path=" << path << endl;
        KDarInteraction ki( this );
        if ( m_browserArchive )
        {
            try
            {
                m_browserArchive->get_children_of( ki, path );
            }
            catch (...)
            {
                kdDebug() << "KDarController::slotGetChildren(): caught exception!" << endl;
            }
        }
    }
}

#include "kdarcontroller.moc"
