/***************************************************************************
 *   Copyright (C) 2004, 2005 by Johnathan Burchill                        *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef _KDARCONTROLLER_H_
#define _KDARCONTROLLER_H_

#include "controller.h"

#include <klibloader.h>
#include <kwallet.h>

#include <qfile.h>
#include <qptrdict.h>

#include <dar/libdar.hpp>

class KDarThread;
class createArchiveThread;
class diffArchiveThread;
class isolateArchiveThread;
class listArchiveThread;
class OpenArchiveThread;
class restoreArchiveThread;
class TestArchiveThread;

class KDarBrowser;
class KDarOpenArchiveDialog;

class QEvent;
class KURL;
//class kdar;
//class KDarConfig;

//class QListViewItem;

/**
 * This is the concrete interface to the KDar controller class. It controls
 * the libdar operations based on user inputs, and is responsible for
 * passing results on to the main
 * view.
 *
 * @short KDar Controller Interface
 * @author Johnathan Burchill, <jkerrb@users.sourceforge.net>
 * @version VERSION
 */

class KDarController : public controller
{
    Q_OBJECT
public:
    KDarController( QObject* parent, const char* name );

    virtual ~KDarController();

    /**
     *  Set the pointer to an open archive. \param name is the name of the archive.
     */
    virtual void setArchive( libdar::archive* archive, const QString & name );
    /**
     *  The currently open archive
     */
    virtual libdar::archive* archive() const;
    /**
     *  closeArchive is a controller method to close an archive
     *  in anticipation of opening a new archive.
     */
    virtual void closeArchive();
    /**
     * Closes the logFile.
     */
    virtual void closeLogFile();
    /**
     *  createArchive is a controller method to create an archive
     *  from filesystem data.
     */
    virtual void createArchive( bool updateListView = true );
    /**
     *  diffArchive is a controller method to compare an archive
     *  against the filesystem.
     */
    virtual void diffArchive();
    /**
     *  isolateArchive is a controller method to isolate an archive
     *  catalog into its own file.
     */
    virtual void isolateArchive();
    /**
     *  listArchive is a controller method to initiate listing the
     *  contents of an archive in the browser window (KListView).
     */
    virtual void listArchive( bool );
    /**
     *  openArchive is a controller method to open an archive.
     */
    virtual void openArchive( const QString & filename = "", bool updateListView = true );
    /**
     * Opens the logFile for appending.
     */
    virtual void openLogFile( const QString & );
    /**
     *  restoreArchive is a controller method to restore the contents
     *  of an archive to the filesystem.
     */
    virtual void restoreArchive();
    /**
     * extract a single inode from the archive
     */
    virtual KURL extractItem( const QString & path );
     /**
     *  testArchive is a controller method to test an archive.
     */
    virtual void testArchive();
    /**
     *  waitForThread checks to see if the given thread type
     *  is running, and waits for it to finish.
     */
    virtual void waitForThread( threadType type );
    /**
     *  customEvent deals with custom events
     */
    void customEvent( QCustomEvent *event );
    /**
     * Writes a message to the logfile.
     */
    virtual void writeLogMessage( QString & );
    /**
     * Cancel a running thread. Use only with a threadsafe libdar.
     */
    virtual bool cancelOperation();
    /**
     * Opens the KDEWallet for storing and retrieving a password.
     */
    virtual bool openKDEWallet();
    /**
     * Obtain a password from a wallet or the user, and store it temporarily
     * in the controller. The stored password can be retrieved
     * with @ref password(). This password should be cleared from memory
     * with @ref clearPassword() once it is not required.
     */
    virtual bool obtainPassword();
    /**
     * Get the password that is temporarily stored in the controller.
     */
    virtual QString password();
    /**
     * Clear the password from memory.
     */
    virtual void clearPassword();
    /**
     * Change the password that is stored in the wallet
     *
     * Returns true if successful, false if not (e.g. user cancelled).
     */
    virtual bool changePassword();
    /**
     * Store a boolean response from user ( for KDarInteraction::pause() )
     */
    virtual void setUserResponse( const bool );
    /**
     * Get the boolean response from the controller
     * ( for KDarInteraction::pause() )
     */
    virtual bool userResponse();
    /**
     * Returns the pointer to the controller's semaphore object.
     * Use this to put a thread to sleep, and to signal it to wake up.
     */
    virtual QSemaphore* semaphore();
    /**
     * Returns whether libdar supports extended attributes
     */
    virtual bool handlesEA() const;
    /**
     * Returns whether libdar supports largefiles (>2GB)
     */
    virtual bool handlesLargefiles() const;
    /**
     * Returns whether libdar supports ext2 nodump feature
     */
    virtual bool handlesNoDump() const;
    /**
     * Returns whether libdar uses Special Memory Allocation scheme
     */
    virtual bool usesSpecialAlloc() const;
    /**
     * Returns size of integer type
     */
    virtual libdar::U_I integerSizeBits() const;
    /**
     * Returns whether libdar is thread-safe
     */
    virtual bool isThreadSafe() const;
    /**
     * Returns whether libdar supports libz compression
     */
    virtual bool handlesZCompression() const;
    /**
     * Returns whether libdar supports libbz2 compression
     */
    virtual bool handlesBZ2Compression() const;
    /**
     * Returns whether libdar supports strong encryption
     */
    virtual bool handlesStrongCrypto() const;
    /**
     * Returns the libdar version string
     */
    virtual const QString libdarVersion() const;

public slots:
    virtual void slotGetChildren( QString & path );

signals:
    void openedArchive( const KURL & archiveName );

private:
    QObject* m_parent;

    KDarBrowser* m_browser;
    createArchiveThread* m_createThread;
    diffArchiveThread* m_diffThread;
    isolateArchiveThread* m_isolateThread;
    listArchiveThread* m_listThread;
    OpenArchiveThread* m_openThread;
    restoreArchiveThread* m_restoreThread;
    TestArchiveThread* m_testThread;

    QFile m_logFile;
    QTextStream m_logStream;

    KWallet::Wallet* m_localWallet;
    /** Temporary holding space for a password. **/
    QString m_password;
    /** Temporary holding space for a boolean response from user. **/
    bool m_userResponse;
    QSemaphore* m_semaphore;
    //Libdar's compiled-in capabilities:
    /** Extended Attributes support**/
    bool m_handlesEA;
    /** Largefile support (>2GB) **/
    bool m_handlesLargefiles;
    /** Handles ext2 filesystem nodump flag **/
    bool m_handlesNoDump;
    /** Special memory allocation scheme **/
    bool m_usesSpecialAlloc;
    /** Integer size ( one of 0 ( infinite ), 32, or 64 ) **/
    libdar::U_I m_integerSizeBits;
    /** Thread-safe library **/
    bool m_isThreadSafe;
    /** libz compression support **/
    bool m_handlesZCompression;
    /** libbz2 compression support **/
    bool m_handlesBZ2Compression;
    /** Strong encryption support **/
    bool m_handlesStrongCrypto;
    /** libdar version string **/
    QString m_libdarVersion;

    /** Pointer to an open browser archive **/
    libdar::archive* m_browserArchive;
    /** Dictionary of open archives **/
    QPtrDict<QString>* m_archives;
    /**
     * A pointer dictionary for running threads. Each thread
     * has a unique pointer, and we associate an archive name to each one.
     */
    QPtrDict<QString>* m_threads;

    /** Pointer to the open archive dialog **/
    KDarOpenArchiveDialog* m_openDialog;
};


#endif // _KDARCONTROLLER_H_
