/***************************************************************************
 *   Copyright (C) 2005 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "kdarinteraction.h"

#include "controller.h"
#include "kdardeclarations.h"

#include "events/kdarprogressevent.h"
#include "events/kdarprogressdialogevent.h"
#include "events/kdarstatusbarevent.h"
#include "events/kdaruserinteractionevent.h"
#include "events/kdaruserinteractionstringevent.h"
#include "events/kdartarlistingevent.h"
#include "events/kdarwarningevent.h"

#include <kdebug.h>
#include <klocale.h>

#include <qapplication.h>
#include <qregexp.h>
#include <qsemaphore.h>
#include <qstring.h>
#include <qtimer.h>

#include <dar/erreurs.hpp>

KDarInteraction::KDarInteraction( controller * aController )
    : libdar::user_interaction(),
      m_controller( aController )
{
    //Tell libdar that we're using our own listing method.
    set_use_listing( true );

    stats = new libdar::statistics( true );
}

KDarInteraction::~KDarInteraction()
{
    if ( stats )
        delete stats;
}

KDarInteraction::KDarInteraction( const KDarInteraction & ki )
    : libdar::user_interaction()
{
    m_controller = ki.m_controller;
}

KDarInteraction & KDarInteraction::operator= ( const KDarInteraction & ki )
{
    if ( this != & ki )
    {
        m_controller = ki.m_controller;
    }
    return *this;
}

libdar::user_interaction * KDarInteraction::clone() const
{
    libdar::user_interaction *ui = new KDarInteraction( *this );
    if ( ui == NULL )
    {
        throw libdar::Ememory( "KDarInteraction::clone" );
    }
    else
    {
        return ui;
    }
}

void KDarInteraction::pause( const std::string & message )
{
//    if ( 1 ) return;
    //Don't access the GUI directly here because we're not the GUI thread.
    QString userQuestion = message.c_str();
    kdarUserInteractionEvent *questionEvent = new kdarUserInteractionEvent( userQuestion );
    //post the questionEvent
    if ( m_controller )
    {
        QApplication::postEvent( m_controller, questionEvent );
        QSemaphore *s = m_controller->semaphore();
        if ( s )
        {
            kdDebug() << "KDarInteraction::pause(): going to sleep..."  << endl;
            (*s)++;
        }
        else
        {
            kdDebug() << "KDarInteraction::pause(): semaphore pointer is 0! Should throw an exception here, eh?" << endl;
        }
        kdDebug() << "KDarInteraction::pause(): waking up!" << endl;
        //What what the user's response?
        bool response = m_controller->userResponse();
        response = true;
        kdDebug() << "KDarInteraction::pause(): User answered " << response << "." << endl;
        //Reset the response to a known value:
        m_controller->setUserResponse( false );
        if ( response == false )
        {
            throw libdar::Euser_abort( message );
        }
    }
}

std::string KDarInteraction::get_string( const std::string & message, bool echo )
{
    //Don't access the GUI directly here because we're not the GUI thread.
    QString question = message.c_str();
    kdarUserInteractionStringEvent *getStringEvent = new kdarUserInteractionStringEvent( question, echo );
    //post the questionEvent
    if ( m_controller )
    {
        QApplication::postEvent( m_controller, getStringEvent );
        QSemaphore *s = m_controller->semaphore();
        if ( s )
        {
            (*s)++;
        }
        else
        {
            kdDebug() << "KDarInteraction::get_string(): semaphore pointer is 0! Should throw an exception here, eh?" << endl;
        }
        kdDebug() << "KDarInteraction::get_string(): waking up!" << endl;

        //The password is stored in m_controller:
        QString response = "";
        response = m_controller->password();
        //Clear the password from the m_controller
        m_controller->clearPassword();
        return response;
    }
    else
    {
        return "";
    }
}

void KDarInteraction::inherited_warning( const std::string & message )
{
    //We have to post a custom event here to display the message because we're not the GUI thread.
    QString userMessage = message.c_str();
    //Implementing a progress bar that shows the percentage of total files
    //that have been processed, as a function of the total number of files
    //to be processed.
    bool sendToStatusBar = false;
    bool sendToProgressDialog = false;
    bool increaseProgressBar = false;
    int field = 0;

    QRegExp addingRegExp = QRegExp( i18n( "libdar create message", "^Adding file to archive: " ) );
    QRegExp savingEARegExp = QRegExp( i18n( "libdar saving EA message", "^Saving Extended Attributes for " ) );
    QRegExp diffOKRegExp = QRegExp( i18n( "libdar diff message", "^OK   " ) );
    QRegExp testOKRegExp = QRegExp( i18n( "libdar test message", "^OK  <ROOT>" ) );
    QRegExp restoringRegExp = QRegExp( i18n( "libdar restore message", "^Restoring file: " ) );
    if ( userMessage.contains( addingRegExp ) )
    {
        userMessage.replace( addingRegExp, i18n( "statusbar replacement for creation message", "Adding " ) );
        sendToProgressDialog = true;
        increaseProgressBar = true;
    }
    else if ( userMessage.contains( savingEARegExp ) )
    {
        userMessage.replace( savingEARegExp, i18n( "statusbar replacement for saving EA message", "Saved EA for " ) );
        sendToProgressDialog = true;
        field = 1;
        increaseProgressBar = false;
    }
    else if ( userMessage.contains( diffOKRegExp ) || userMessage.contains( testOKRegExp ) )
    {
        sendToStatusBar = true;
        increaseProgressBar = true;
    }
    else if ( userMessage.contains( restoringRegExp ) )
    {
        userMessage.replace( restoringRegExp, i18n( "statubar replacement for restore message", "Restoring " ) );
        sendToStatusBar = true;
        increaseProgressBar = true;
    }
    if ( m_controller )
    {
        if ( sendToStatusBar )
        {
            //Post the statusBar event
            kdarStatusBarEvent *statusEvent = new kdarStatusBarEvent( userMessage, FOREVER );
            QApplication::postEvent( m_controller, statusEvent );
        }
        else if ( sendToProgressDialog )
        {
            //Post the progressDialog event
            kdarProgressDialogEvent *progressEvent = new kdarProgressDialogEvent( userMessage, field );
            QApplication::postEvent( m_controller, progressEvent );
        }
        else
        {
            kdarWarningEvent *warnEvent = new kdarWarningEvent( userMessage );
            QApplication::postEvent( m_controller, warnEvent );
        }
        if ( increaseProgressBar )
        {
            //increase the progress counter
            kdarProgressEvent *addEvent = new kdarProgressEvent( kdarProgressEvent::PROGRESS_INCREASE_ADDED );
            QApplication::postEvent( m_controller, addEvent );
        }
    }
    else
    {
        kdDebug() << "KDarInteraction::inherited_warning(): " << userMessage << endl;
    }
}

void KDarInteraction::listing( const std::string & flag, const std::string & perm, const std::string & uid, const std::string & gid, const std::string & size, const std::string & date, const std::string & filename, bool is_dir, bool has_children )
{
    //We have to post a custom event here to display the listing because we're not the GUI thread.
    QString qflag = flag.c_str();
    QString qperm = perm.c_str();
    QString quid = uid.c_str();
    QString qgid = gid.c_str();
    QString qsize = size.c_str();
    QString qdate = date.c_str();
    QString qfilename = filename.c_str();
    bool qisdir = is_dir;
    bool qhaschildren = has_children;

    //create and post a new tarListingEvent
    kdarTarListingEvent *tarListingEvent = new kdarTarListingEvent( qflag, qperm, quid, qgid, qsize, qdate, qfilename, qisdir, qhaschildren );
    if ( m_controller )
    {
        QApplication::postEvent( m_controller, tarListingEvent );
    }
}

void KDarInteraction::checkProgress()
{
    kdDebug() << "KDarInteraction::checkProgress()" << endl;
    if ( stats )
    {
        kdDebug() << "KDarInteraction::checkProgress(): stats exists." << endl;
    }
}

