/***************************************************************************
 *   Copyright (C) 2005 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "kdarconfigdialog.h"

//Config module
#include "kdarsettings.h"
#include "kdarconfig.h"
#include "kdarfilterconfigdialog.h"

#include <klibloader.h>
#include <klocale.h>

#include <qstringlist.h>
#include <qvaluelist.h>

#include <iostream>

KDarConfigDialog::KDarConfigDialog( QWidget* parent )
    : KConfigDialog( parent, "settings", KDarSettings::self() ),
      m_config( 0 ),
      m_isValid( 0 )
{
    //Load the kcm_kdar module:
    const KLibrary *lib = KLibLoader::self()->library( "kcm_kdar" );
    if ( lib )
    {
        //Try to get a new KDarConfig*
        if ( lib->hasSymbol( "create_kdarconfig" ) )
        {
            m_config = (KDarConfig *) ( ( KDarConfig * (*) ( QWidget*, const char* ) ) (lib->symbol( "create_kdarconfig" ))) ( parent, "kdarconfig");
        }
    }
    if ( m_config )
    {
        //We got a new KDarConfig*, so let's add pages to the dialog:
        addPage( m_config->generalDialog(), i18n( "General" ), "kdar" );
        addPage( m_config->sliceDialog(), i18n( "Slicing" ), "kdar_slicing" );
        addPage( m_config->cryptographyDialog(), i18n( "Cryptography" ), "lock" );
        addPage( m_config->compressionDialog(), i18n( "Compression" ), "compression" );
        addPage( m_config->directoryDialog(), i18n( "Directory filters" ), "kdar_directory" );
        addPage( m_config->fileDialog(), i18n( "File filters" ), "files" );
        addPage( m_config->filesystemDialog(), i18n( "Filesystem" ), "filesystem" );
        addPage( m_config->restoreDialog(), i18n( "Restoring" ), "kdar_restore" );
        addPage( m_config->compareDialog(), i18n( "Time shifting" ), "kdar_compare" );
        addPage( m_config->fontDialog(), i18n( "Fonts" ), "fonts" );
        connect( m_config, SIGNAL( widgetChanged() ), this, SLOT( updateSettings() ) );
        m_isValid = true;
    }
}

KDarConfigDialog::~KDarConfigDialog()
{
    if ( m_config ) delete m_config;
}

void KDarConfigDialog::updateSettings()
{
    //If the slice media type changed, we need to update the
    //config with the new "current" slice and first slice sizes.
    //This works in the following way:
    //1. libdar operations will get the slice sizes from KDarSettings.
    //2. When the user selects a predefined media type, the
    //   slice size is set from a const QStringList. The first slice size
    //   is set from a QStringList. These settings are written to disk
    //   by the KConfigDialog's KConfigDialogManager.
    //3. When the user changes the first slice size of a predefined
    //   media type, the corresponding entry in the QStringList is set.
    //4. When the user changes the slice size of a predefined media type,
    //   the type is set to custom and the custom slice size is set.
    //   The custom first slice size is set from the config.

    //The predefined media types are stored in the config file.
    //They have no corresponding widget.
    //Read the slice sizes from the config file:
    int media = m_config->mediaType();
    QStringList firstSizes = KDarSettings::mediaTypeFirstSizes();
    QValueList<int> firstSuffices = KDarSettings::mediaTypeFirstSizeSuffices();
    //Update the first slice size of the current media type:
    firstSizes[ media ] = m_config->firstSliceSize();
    firstSuffices[ media ] = m_config->firstSliceSizeSuffix();
    //Set the first slice sizes:
    KDarSettings::setMediaTypeFirstSizes( firstSizes );
    KDarSettings::setMediaTypeFirstSizeSuffices( firstSuffices );
    //Update the slice size if the media type is custom:
    if ( media == KDarSettings::EnumMediaType::Custom )
    {
        QStringList sizes = KDarSettings::mediaTypeSizes();
        QValueList<int> suffices = KDarSettings::mediaTypeSizeSuffices();
        sizes[ KDarSettings::EnumMediaType::Custom ] = m_config->sliceSize();
        suffices[ KDarSettings::EnumMediaType::Custom ] = m_config->sliceSizeSuffix();
        //Set the slice sizes:
        KDarSettings::setMediaTypeSizes( sizes );
        KDarSettings::setMediaTypeSizeSuffices( suffices );
    }

    //Store filters
    //compression filters
    KDarFilterConfigDialog* compressionDlg = (KDarFilterConfigDialog*) m_config->compressionDialog();
    if ( compressionDlg )
    {
        std::cout << "Saving compression filters!!!!" << std::endl;
        KDarSettings::setCompressionFilters( compressionDlg->getFilters() );
    }
    //directory filters
    KDarFilterConfigDialog* directoryDlg = (KDarFilterConfigDialog*) m_config->directoryDialog();
    if ( directoryDlg )
    {
        std::cout << "Saving directory filters!!!!" << std::endl;
        KDarSettings::setDirectoryFilters( directoryDlg->getFilters() );
    }
    //files filters
    KDarFilterConfigDialog* fileDlg = (KDarFilterConfigDialog*) m_config->fileDialog();
    if ( fileDlg )
    {
        std::cout << "Saving file filters!!!!" << std::endl;
        KDarSettings::setFileFilters( fileDlg->getFilters() );
    }
    KDarSettings::writeConfig();

    std::cout << "KDarConfigDialog::updateSettings(): done." << std::endl;

}

void KDarConfigDialog::updateWidgets()
{
    //This is where we update the filter listViews
    //compression filters
    KDarFilterConfigDialog* compressionDlg = (KDarFilterConfigDialog*) m_config->compressionDialog();
    if ( compressionDlg )
    {
        compressionDlg->setFilters( KDarSettings::compressionFilters() );
    }
    //directory filters
    KDarFilterConfigDialog* directoryDlg = (KDarFilterConfigDialog*) m_config->directoryDialog();
    if ( directoryDlg )
    {
        directoryDlg->setFilters( KDarSettings::directoryFilters() );
    }
    //file filters
    KDarFilterConfigDialog* fileDlg = (KDarFilterConfigDialog*) m_config->fileDialog();
    if ( fileDlg )
    {
        fileDlg->setFilters( KDarSettings::fileFilters() );
    }

}

#include "kdarconfigdialog.moc"
