
#ifndef _KDARPART_H_
#define _KDARPART_H_

#include <kparts/part.h>
#include <kparts/factory.h>

class KDarActionView;
class KDarBrowser;
class KDarManager;
class KDarMessages;
class KDarLog;

class KDarConfig;
class controller;
class ArchiveManager;

class KAboutDialog;
class KAction;
class KActionMenu;
class KDockArea;
class KListView;
class KPrinter;
class KRecentFilesAction;
class KToggleAction;
class KURL;

class QListViewItem;
class QPainter;
class QPoint;
class QPopupMenu;
class QString;

/**
 * This is a "Part".  It that does all the real work in a KPart
 * application.
 *
 * @short Main Part
 * @author Johnathan Burchill <jkerrb@shaw.ca>
 * @version 0.1
 */
class KDarPart : public KParts::ReadOnlyPart
{
    Q_OBJECT
public:
    /**
     * Default constructor
     */
    KDarPart(QWidget *parentWidget, const char *widgetName,
                    QObject *parent, const char *name);

    /**
     * Destructor
     */
    virtual ~KDarPart();

    /**
     * accessor for the actions view
     */
    virtual QWidget* actionsDockWidget();
    /**
     * accessor for the browser
     */
    virtual QWidget* browserDockWidget();
    /**
     * accessor for the manager
     */
    virtual QWidget* managerDockWidget();
    /**
     * accessor for the message window
     */
    virtual QWidget* messagesDockWidget();
    /**
     * accessor for the log window
     */
    virtual QWidget* logDockWidget();
    /**
     * accessor for the dockHideShow popup menu
     */
    virtual QPopupMenu* dockPopupMenu();

    /**
     * set the state of the cancel action
     */
    virtual void setCancelState( bool state );
    /**
     * add some text to the message window
     */
    virtual void appendMessage( const QString & message );
    /**
     * set the status bar message
     */
    virtual void setStatusBarMessage( const QString & message, const int duration );

protected:
    /**
     * Get an instance of a controller
     */
    void getController();
    /**
     * Open an archive file
     */
    virtual bool openFile();
    /**
     * Creates actions and connects them to slots
     */
    void setupActions();
    /**
     * Example configuration
     */
    void readConfig();
    void saveSettings();
    void saveDockStatus();
    void restoreDockStatus();

public slots:
    /**
     * clear the status bar of messages.
     */
    virtual void clearStatusBarMessages();
    /**
     * show the custom about dialog
     */
    virtual void showAboutDialog();
    /**
     * Add an archive to the recent files list
     */
    void addRecentArchive( const KURL & archiveName );
    /**
     * Remove an archive from the recent files list
     */
    void removeRecentArchive( const KURL & archiveName );
    /**
     * Open the archive
     */
    void openArchive( const KURL & archiveName );

protected slots:
    /**
     * Show the browser's context menu
     */
    void showBrowserPopupMenu( KListView*, QListViewItem*, const QPoint & );
    /**
     * Show the manager's context menu
     */
    void showManagerPopupMenu( KListView*, QListViewItem*, const QPoint & );
    /**
     * Print the contents of the browser
     */
    void printBrowser();
    /**
     * Print the contents of the manager
     */
    void printManager();
    /**
     * Print the contents of the message window
     */
    void printMessages();
    /**
     * Print the contents of the log window
     */
    void printLog();
    /**
     * Configure preferences
     */
    void optionsPreferences();
    /**
     * Print messages
     */
    void filePrint();
    /**
     * Clear message window
     */
    void clearMessages();
    /**
     * Clear log window
     */
    void clearLog();
    /**
     * Cancel an archive operation
     */
    void cancelOperation();
    /**
     * Perform full backup of filesystem
     */
    void fullBackup();
    /**
     * Perform a differential backup of filesystem based on
     * most recent full or differential backup.
     */
    void diffBackup();
    /**
     * Perform full restore of an archive onto the filesystem
     */
    void fullRestore();
    /**
     * Create a database
     */
    void managerNewDatabase();
    /**
     * Add an archive to a database
     */
    void managerAddArchive();
    /**
     * Remove an archive from the manager
     */
    void managerRemoveArchive();
    /**
     * Search the managed archives for a file or pattern
     */
    void managerSearchManagedArchives();
    /**
     * Choose an archive to open
     */
    void chooseArchive();
    /**
     * Display the contents of an archive in the browser window
     */
    void listArchive();
    /**
     * Create an archive from a filesystem
     */
    void createArchive();
    /**
     * Restore an archive onto a filesystem
     */
    void restoreArchive();
    /**
     * Isolate contents of archive into separate file
     */
    void isolateArchive();
    /**
     * compare contents of archive with filesystem
     */
    void diffArchive();
    /**
     * Test the integrity of an archive
     */
    void testArchive();

private:
    KDockArea* m_dockWindow;
    KDarActionView* m_actions;
    KDarBrowser* m_browser;
    KDarManager* m_manager;
    KDarMessages* m_messages;
    KDarLog* m_log;

    KDarConfig* m_kdarConfig;
    controller* m_controller;

    ArchiveManager* m_archiveManager;

    KPrinter *m_printer;
    KAction *m_cancel;
/*    KToggleAction *m_logWindowAction;*/
    KRecentFilesAction *m_recentFilesAction;
    KActionMenu *browserPopupMenu;
    KToggleAction *m_showPreviewsAction;
    KActionMenu *managerPopupMenu;
//    KActionMenu *messagesPopupMenu;

    //Config settings
    bool m_verboseOutput;
    bool m_useCreateArchiveWizard;
    KAboutDialog* m_aboutDialog;

};

class KInstance;
class KAboutData;

class KDarPartFactory : public KParts::Factory
{
    Q_OBJECT
public:
    KDarPartFactory();
    virtual ~KDarPartFactory();
    virtual KParts::Part* createPartObject( QWidget *parentWidget, const char *widgetName, QObject *parent, const char *name, const char *classname, const QStringList &args );
    static KInstance* instance();

private:
    static KInstance* s_instance;
    static KAboutData* s_about;

};

#endif // _KDARPART_H_
