/***************************************************************************
 *   Copyright (C) 2005 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "kdarbrowser.h"

#include "kdararchiveitem.h"
#include "kdarlistviewitem.h"

#include "../controllers/controller.h"
#include "../controllers/kdarinteraction.h"
#include "../kcm_modules/kdarsettings.h"

#include <kactionclasses.h>
#include <kdebug.h>
#include <kdeversion.h>
#include <kiconloader.h>
#include <kimagefilepreview.h>
#include <klistview.h>
#include <klistviewsearchline.h>
#include <klocale.h>
#include <kmimetype.h>

#include <qcheckbox.h>
#include <qdir.h>
#include <qfile.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpoint.h>
#include <qpopupmenu.h>
#include <qregexp.h>
#include <qsplitter.h>
#include <qtooltip.h>
#include <qvbox.h>
#include <qwhatsthis.h>

KDarBrowser::KDarBrowser( QWidget* parent, const char * name )
    : QWidget( parent, name ),
      m_controller( 0 ),
      m_previousURL( "" )
{
    QGridLayout* mainlayout = new QGridLayout( this );
    mainlayout->setAutoAdd( true );

    QSplitter* vSplitter = new QSplitter( Qt::Vertical, this, "Archive View Splitter" );
    QSplitter* hSplitter = new QSplitter( vSplitter, "Preview Splitter" );
    QVBox* browserVBox = new QVBox( hSplitter );
    //Set up the KListView:
    m_browserView = new KListView( browserVBox, "Archive Browser" );
    m_browserView->addColumn( i18n( "Filename" ) );
    m_browserView->addColumn( i18n( "Permissions" ) );
    m_browserView->addColumn( i18n( "User ID" ) );
    m_browserView->addColumn( i18n( "Group ID" ) );
    m_browserView->addColumn( i18n( "Size" ) );
    m_browserView->addColumn( i18n( "Date" ) );
    m_browserView->addColumn( i18n( "Status" ) );
    m_browserView->setMouseTracking( true );
    m_browserView->setAcceptDrops( true );
    m_browserView->setSelectionModeExt( KListView::FileManager );
    m_browserView->setAllColumnsShowFocus( true );
    m_browserView->setShowSortIndicator( true );
    m_browserView->setRootIsDecorated( true );
    m_browserView->setResizeMode( KListView::LastColumn );
    //Sort by filetype first
    m_browserView->setSorting( 1 );
    //Pass on the expanded signal
    connect( m_browserView, SIGNAL( expanded( QListViewItem* ) ), this, SLOT( slotItemExpanded( QListViewItem* ) ) );
    //Pass on the contextMenu signal
    connect( m_browserView, SIGNAL( contextMenu( KListView*, QListViewItem*, const QPoint&) ), this, SLOT( slotContextMenu( KListView*, QListViewItem*, const QPoint& ) ) );

    m_currentItem = 0;

#if KDE_VERSION >= KDE_MAKE_VERSION(3,4,0)
    //A searchline widget for the listview
    m_searchWidget = new KDarBrowserSearchWidget( m_browserView, browserVBox, "BrowserSearchWidget" );
#endif

    m_preview = new KImageFilePreview( hSplitter );
    m_preview->setShown( KDarSettings::showBrowserPreviews() );
//     connect( m_browserView, SIGNAL( clicked( QListViewItem * ) ), this, SLOT( showPreview( QListViewItem * ) ) );
    connect( m_browserView, SIGNAL( currentChanged( QListViewItem * ) ), this, SLOT( showPreview( QListViewItem * ) ) );

    //This is used for keeping track of opened archives.
    m_archiveView = new KListView( vSplitter, "Archive View" );
    m_archiveView->addColumn( i18n( "Archive" ) );
    m_archiveView->addColumn( i18n( "Root" ) );
    m_archiveView->addColumn( i18n( "Status" ) );
    m_archiveView->setSorting( 0 );
    m_archiveView->setItemMargin( 2 );
    m_archiveView->setResizeMode( KListView::LastColumn );
    connect( m_archiveView, SIGNAL( doubleClicked( QListViewItem*, const QPoint &, int ) ), this, SLOT( setShownArchive( QListViewItem*, const QPoint &, int ) ) );

}

KDarBrowser::~KDarBrowser()
{
    //Clean up temporary preview file and directories
    removePreviewURL( m_previousURL );
}

void KDarBrowser::addBrowserEntry( QString filename, QString permissions, QString uid, QString gid, QString size, QString date, QString status, bool isDirectory, bool hasChildren )
{
    KURL url = "/" + filename;
    QPixmap icon = KMimeType::pixmapForURL( url, 0, KIcon::Small );
    if ( isDirectory )
    {
        icon = SmallIcon( "folder" );
    }
    //remove "<empty>" from the current directory, if it exists
/*    kdarListViewItem * firstItem = directory->firstChild();
    if ( firstItem )
    {
        if ( firstItem->text( 0 ) == "<empty>" )
        {
            delete firstItem;
            firstItem = 0;
        }
    }
*/
    kdarListViewItem *newItem = 0;
    if ( ! m_currentItem )
    {
        newItem =  new kdarListViewItem( m_browserView, filename, permissions, uid, gid, size, date, status );
    }
    else
    {
        newItem =  new kdarListViewItem( m_currentItem, filename, permissions, uid, gid, size, date, status );
    }
    newItem->setPixmap( 0, icon );
    //give this item an "<empty>" child if is a directory and has children
    if ( isDirectory && hasChildren )
    {
        (void) new kdarListViewItem( newItem, "<empty>" );
    }
}

void KDarBrowser::clearBrowserView()
{
    if ( m_browserView )
    {
        m_browserView->clear();
        m_currentItem = 0;
        m_preview->clearPreview();
    }
}

void KDarBrowser::slotItemExpanded( QListViewItem* item )
{
    kdarListViewItem* clicked = static_cast<kdarListViewItem*> ( item );
    if ( clicked )
    {
        m_currentItem = clicked;
//        QString path = clicked->text( 0 );
        QString path = clicked->path();
//         kdDebug() << "KDarBrowser::slotItemExpanded(): path = " << path << endl;
        kdarListViewItem* child = clicked->firstChild();
        if ( child )
        {
            if ( child->text( 0 ) == "<empty>" )
            {
                delete child;
                emit childrenRequested( path );
            }
        }
    }
}

void KDarBrowser::slotContextMenu( KListView* view, QListViewItem* item, const QPoint& point )
{
    if ( item )
    {
        emit contextMenu( view, item, point );
    }
}

void KDarBrowser::showPreview( QListViewItem* item )
{
//     kdDebug() << "KDarBrowser::showPreview()" << endl;
    kdarListViewItem* entry = static_cast<kdarListViewItem*>(item);
    if ( entry )
    {
        if ( KDarSettings::showBrowserPreviews() && !entry->firstChild() && m_controller )
        {
//             kdDebug() << "KDarBrowser::showPreview(): extracting item \"" << entry->path() << "\"" << endl;
            //Only extract a preview if not a folder
            KURL url = m_controller->extractItem( entry->path() );
//             kdDebug() << "KDarBrowser::showPreview(): url is " << url.path() << endl;
            if ( !url.isEmpty() )
            {
                m_preview->showPreview( url );
                //remove the previous URL
                removePreviewURL( m_previousURL );
                m_previousURL = url;
            }
        }
    }
}

void KDarBrowser::togglePreviews( bool show )
{
    m_preview->setShown( show );
    KDarSettings::setShowBrowserPreviews( show );
    KDarSettings::writeConfig();

}

void KDarBrowser::removePreviewURL( const KURL & url )
{
//     kdDebug() << "KDarBrowser::showPreview(): previousURL.path() = " << url.path() << endl;
    if ( QFile::exists( url.path() ) )
    {
        QFile file( url.path() );
//         kdDebug() << "KDarBrowser::showPreview(): removing previous file: " << file.name() << endl;
        QFile::remove( file.name() );
        QDir directory( url.directory() );
        bool removed = true;
        while ( removed )
        {
            QString directoryName = directory.dirName();
//             kdDebug() << "Directory.dirName() = " << directoryName << endl;
            bool cdUp = directory.cdUp();
//             kdDebug() << "cdUp = " << cdUp << endl;
            if ( cdUp )
            {
//                 kdDebug() << "-->directory.path() = " << directory.path() << endl;
                removed = directory.rmdir( directoryName );
//                 kdDebug() << "removed = " << removed << endl;
            }
            else
            {
                removed = false;
            }
        }
    }
}

void KDarBrowser::setShownArchive( QListViewItem* item, const QPoint &, int )
{
    KDarArchiveItem* archiveItem = static_cast<KDarArchiveItem*>( item );
    if ( !archiveItem )
        return;

    if ( m_controller )
        m_controller->setArchive( archiveItem->archive(), archiveItem->name() );
    clearBrowserView();
    removePreviewURL( m_previousURL );
    m_previousURL = "";
    //Get child item from archive.
    KDarInteraction userInteractionDialog( m_controller );
    archiveItem->archive()->get_children_of( userInteractionDialog, "" );

}

void KDarBrowser::addArchive( libdar::archive* arch, const QString & path, const QString & displayedName )
{
    if ( arch && !path.isEmpty() && !displayedName.isEmpty() )
        (void) new KDarArchiveItem( arch, path, m_archiveView, displayedName );
}


//The search widget:
KDarBrowserSearchWidget::KDarBrowserSearchWidget( KListView* lv, QWidget* parent, const char* name )
    : QHBox( parent, name )
{
    setSpacing( 5 );
    setFrameStyle( QFrame::StyledPanel );

    QLabel* label = new QLabel( i18n( "Filter:" ), this, "SearchLabel" );
    QToolTip::add( label, i18n ( "Filter the displayed archive items." ) );
    m_searchline = new KDarBrowserSearchLine( this, lv );
    QWhatsThis::add( m_searchline, i18n ( "Use this to filter the displayed archive items. This does not perform a complete archive search. It only applies to folders which you have already browsed." ) );

//     m_globCheckBox = new QCheckBox( i18n( "filter uses globbing", "Wildcards" ), this );
//     QToolTip::add( m_globCheckBox, i18n( "Filter using RegExp or wildcards." ) );
//     m_globCheckBox->setChecked( KDarSettings::browserSearchLineUsesGlobs() );
//     connect( m_globCheckBox, SIGNAL( toggled( bool ) ), m_searchline, SLOT( setGlobbing( bool ) ) );


}

//The searchline
KDarBrowserSearchLine::KDarBrowserSearchLine( QWidget* parent, KListView* listView, const char* name )
    : KListViewSearchLine( parent, listView, name ),
      menu( 0 )
{
    m_usingGlobs = KDarSettings::browserSearchLineUsesGlobs();
    m_useGlobsAction = new KToggleAction( i18n( "&Filter using wildcards..." ), "filter", 0, this, "filter_using_globs" );
    connect( m_useGlobsAction, SIGNAL( toggled( bool ) ), this, SLOT( setGlobbing( bool ) ) );
    m_useGlobsAction->setChecked( KDarSettings::showBrowserPreviews() );
//     createPopupMenu();

}

KDarBrowserSearchLine::~KDarBrowserSearchLine()
{
/*    if ( menu )
        delete menu;*/
}

void KDarBrowserSearchLine::setGlobbing( bool use )
{
    KDarSettings::setBrowserSearchLineUsesGlobs( use );
    KDarSettings::writeConfig();
    m_usingGlobs = use;
    updateSearch();
}

QPopupMenu* KDarBrowserSearchLine::createPopupMenu()
{
    if ( !menu )
    {
        menu = new KActionMenu( "Browser Search Line", this );
        menu->insert( m_useGlobsAction );
    }
    return (QPopupMenu*) menu->popupMenu();
}

//Here is the search widget match function:
bool KDarBrowserSearchLine::itemMatches( const QListViewItem* item, const QString & filter ) const
{
    if ( !item )
        return false;
    //Based on the KDE library source code:
    if( filter.isEmpty() )
    {
        return true;
    }
    QRegExp re( filter );
    re.setCaseSensitive( true );
    re.setWildcard( m_usingGlobs );
    QString s( item->text( 0 ) );
    if ( m_usingGlobs )
    {
        if ( ( re.search( s ) != -1 ) && ( re.matchedLength() == (int) s.length() ) )
        {
            return true;
        }
    }
    else
    {
        if ( re.search( s ) != -1 )
        {
            return true;
        }
    }
    return false;
}

#include "kdarbrowser.moc"
