/***************************************************************************
 *   Copyright (C) 2005 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "kdararchive.h"
#include "../controllers/kdarinteraction.h"

#include <klocale.h>
#include <kmessagebox.h>
#include <kurl.h>

#include <qapplication.h>
#include <qfileinfo.h>
#include <qregexp.h>
#include <qstring.h>

#include <netinet/in.h>
#include <sys/types.h>

KDarArchive::KDarArchive( const QString & filename )
    : m_path( filename ),
      m_date( QDateTime() )

{
    m_darFile.setName( filename );
    if ( !filename.isEmpty() && openArchive() )
    {
        //Read in the header
        //Read in the header version
        KDarInteraction info;
        libdar::fichier tmpFile( info, m_darFile.handle() );
        m_header.read( tmpFile );
        m_headerVersion.read( tmpFile );
        m_darFile.close();
        //Date of creation
        QFileInfo fileInfo( m_darFile );
        m_date = fileInfo.created();
    }
}

KDarArchive::~KDarArchive()
{
    if ( m_darFile.isOpen() )
        m_darFile.close();
}

bool KDarArchive::openArchive()
{
    if ( m_darFile.exists() )
    {
        if ( m_darFile.open( IO_ReadOnly ) )
        {
            return true;
        }
        else
        {
            //display error message!
            KMessageBox::sorry( 0, i18n( "Could not open file for reading: %1" ).arg( qApp->translate( "QFile", m_darFile.errorString() ) ), i18n( "Open failed" ) );
            return false;
        }
    }
    else
    {
        //display error message!
        KMessageBox::sorry( 0, i18n( "%1: No such file or directory" ).arg( m_darFile.name() ), i18n( "Open failed" ) );
        return false;
    }
}

QDateTime & KDarArchive::date()
{
    if ( m_date.isNull() )
    {
        QFileInfo fileInfo( m_darFile );
        m_date = fileInfo.created();
    }
    return m_date;
}

void KDarArchive::setDate( const QString & d )
{
    if ( !d.isNull() )
    {
        m_date = QDateTime::fromString( d );
    }
}

QString KDarArchive::simpleDate() const
{
    QString dateFormat = i18n( "archive date format", "d MMM yyyy" );
    return m_date.toString( dateFormat );
}

QString KDarArchive::path()
{
    return m_path.path();
}

void KDarArchive::setPath( const QString & p )
{
    if ( !p.isNull() )
    {
        m_path.setPath( p );
    }
}

bool KDarArchive::isDarArchive() const
{
    if ( m_header.magic == libdar::SAUV_MAGIC_NUMBER )
        return true;
    else
        return false;
}

const QString & KDarArchive::internalName() const
{
    return m_internalName;
}

void KDarArchive::setInternalName( const QString & n )
{
    if ( n.isEmpty() )
    {
        QString result;
        unsigned char num;
        libdar::U_I read;
        for ( read = 0; read < libdar::LABEL_SIZE; ++read )
        {
            num = m_header.internal_name[ read ];
            QString tmp;
            result += tmp.sprintf( "%X", num );
        }
        m_internalName = result;
    }
}

QString KDarArchive::darVersion() const
{
    return QString( m_headerVersion.edition );
}

int KDarArchive::compressionAlgorithm() const
{
    return m_headerVersion.algo_zip;
}

QString KDarArchive::compressionAlgorithmName() const
{
    switch( m_headerVersion.algo_zip )
    {
        case libdar::none:
            return i18n( "compression type", "none" );
            break;
        case libdar::zip:
            return i18n( "compression type", "zip" );
            break;
        case libdar::gzip:
            return i18n( "compression type", "gzip" );
            break;
        case libdar::bzip2:
            return i18n( "compression type", "bzip2" );
            break;
        default:
            return i18n( "compression type", "unknown" );
    }
}

QString KDarArchive::commandLine() const
{
    return QString( m_headerVersion.cmd_line );
}

QString KDarArchive::flag() const
{
    return QString::number( m_headerVersion.flag );
}

bool KDarArchive::isEncrypted() const
{
    return ( m_headerVersion.flag & VERSION_FLAG_SCRAMBLED );
}

bool KDarArchive::isRootEASaved() const
{
    return ( m_headerVersion.flag & VERSION_FLAG_SAVED_EA_ROOT );
}

bool KDarArchive::isUserEASaved() const
{
    return ( m_headerVersion.flag & VERSION_FLAG_SAVED_EA_USER );
}

QString KDarArchive::pathname( const KURL & filename )
{
    return filename.directory( false );
}

QString KDarArchive::pathname()
{
    return pathname( m_path );
}

QString KDarArchive::basename( const KURL & filename )
{
    QRegExp expression( EXTENSIONREGEXPSTRING );
    expression.match( filename.filename() );
    return expression.cap( 1 );
}

QString KDarArchive::basename()
{
    return basename( m_path );
}

QString KDarArchive::pathBasename( const KURL & filename )
{
    return pathname( filename ) + basename( filename );
}

QString KDarArchive::pathBasename()
{
    return pathBasename( m_path );
}

//#include ".moc"
