/***************************************************************************
 *   Copyright (C) 2005 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _KDARARCHIVE_H_
#define _KDARARCHIVE_H_

#include <kurl.h>

#include <qdatetime.h>
#include <qfile.h>

#include <string>

#include <dar/libdar.hpp>
#include "header.hpp"
#include <dar/header_version.hpp>

//From <dar/header_verion.hpp>
    const int VERSION_FLAG_SAVED_EA_ROOT = 0x80;
    const int VERSION_FLAG_SAVED_EA_USER = 0x40;
    const int VERSION_FLAG_SCRAMBLED     = 0x20;      // scrambled or strong encryption used

/**
 * This class provides access to archive information from the slice and
 * archive headers.
 */

    const QString EXTENSION( "dar" );
    const QString EXTENSIONREGEXPSTRING( "(.*)(\\.[1-9][0-9]*\\.dar$)" );

class KDarArchive
{
public:

    KDarArchive( const QString & filename = QString::null );

    ~KDarArchive();

    /** opens the archive file for reading the headers... **/
    bool openArchive();
    /** just the basename minus the slice number and extension **/
//     QString basename() const;
    /** date that archive was created **/
    QDateTime & date();
    /** set the date that archive was created **/
    void setDate( const QString & d );
    /** date as a string (DD Mon YYYY ) **/
    QString simpleDate() const;
    /** path to the archive **/
    QString path();
    /** set the to the archive **/
    void setPath( const QString & p );
    /** Notes that were set by user **/
    const QString & notes() const { return m_notes; };
    /** Set user notes **/
    void setNotes( const QString & userNotes ) { m_notes = userNotes; };
    /** dar_manager database **/
    const QString & database() const { return m_database; };
    /** set the database **/
    void setDatabase( const QString & database ) { m_database = database; };
    /**
     * is the archive a valid DAR slice? Checks header magic number
     * which should be 123
     */
    bool isDarArchive() const;
    /** the internal name of the archive in hexadecimal **/
    const QString & internalName() const;
    /** set the internal name **/
    void setInternalName( const QString & n = QString::null );
    /** version of archive format **/
    QString darVersion() const;
    /** compression algorithm **/
    int compressionAlgorithm() const;
    /** name of compression algorithm **/
    QString compressionAlgorithmName() const;
    /** the command line used to create the archive (not used anymore?) **/
    QString commandLine() const;
    /** the archive flag: includes whether EA is used and encryption type **/
    QString flag() const;
    /** whether the archive is encrypted **/
    bool isEncrypted() const;
    /** whether the archive contains root extended attributes**/
    bool isRootEASaved() const;
    /** whether the archive contains user extended attributes**/
    bool isUserEASaved() const;
    /**
     * returns the path to the archive, not including the basename
     * always returns with a trailing slash
     */
    static QString pathname( const KURL & filename );
    /**
     * returns path to archive
     */
    QString pathname();
    /**
     * returns the basename of the archive, not including the path,
     * and without the trailing slice number and extension (i.e., .1.dar)
     */
    static QString basename( const KURL & filename );
    /**
     * returns basename
     */
    QString basename();
    /**
     * returns the full path and basename of the archive,
     * without the trailing slice number and extension
     */
    static QString pathBasename( const KURL & filename );
    /**
     * returns basename including full path
     */
    QString pathBasename();

protected:

    /** QFile to open, read data, etc. **/
    QFile m_darFile;
    /** User notes **/
    QString m_notes;

private:

    /** path to archive **/
    KURL m_path;
    /** creation date **/
    QDateTime m_date;
    /** header info **/
    libdar::header m_header;
    /** archive header info **/
    libdar::header_version m_headerVersion;
    /** internal name (this is assigned by libdar) **/
    QString m_internalName;
    /** dar_manager database **/
    QString m_database;

};

#endif // _KDARARCHIVE_H_
